

         /*********************************************************/
         /*                                                       */
         /*                 COPYRIGHT (C) 1988 BY                 */
         /* CAD Group of Dipartimento di Automatica e Informatica */
         /*             Politecnico di Torino, Italy.             */
         /*                  ALL RIGHTS RESERVED.                 */
         /*                                                       */
         /* This software is furnished under a license and may be */
         /* used and copied  only in accordance with the terms of */
         /* such license. Unauthorized copies will be prosecuted. */
         /*                                                       */
         /*********************************************************/





/******************************************************************************

    Filename:         portable.c

    Author:           Pier Luca Montessoro

    Creation date:    26 February 1988

    Contents:         This file contains some critical functions for the 
                      software portability.


                          *** DOCUMENTATION ***

      This file contains some critical functions for the software portability.
 Conditional compilation allows to make them machine-independent.


 NO_MALLOC

      is the 16-bit machines memory allocation function. The main problem is 
 that on 16-bit machines, the calloc, malloc, and other memory allocation 
 functions that use unsigned int parameters, give heavy limitations in the 
 size of the memory to be allocated. So, a memory pool is statically 
 allocated and never released. Typically the mem_alloc function is compiled 
 only for the 16-bit machines. For the other machines the standard malloc 
 function is used (see env.h).


 NO_FREE

      is the function to release the memory allocated using the no_malloc one.
 The no_free function is compiled only for the 16-bit machines. For the 
 other machines the standard free function is used (see env.h).


 NO_CFREE

      is conditionally compiled for the systems that does not support the 
 cfree function in its libraries.


 NO_TIMES

      is conditionally compiled for the systems that does not support the 
 times function in its libraries. It stores the total elapsed time from the 
 start of the program in the time_struct data structure. 


 NO_TOLOWER

      is conditionally compiled for the systems that does not support the 
 tolower function in the standard libraries.


 NO_TOUPPER

      is conditionally compiled for the systems that does not support the 
 toupper function in the standard libraries.


 NO_SLEEP

      to wait for a while.


 ******************************************************************************/



#include "env.h"



/******************************************************************************

    Function name:    no_malloc

    Authors:          Pier Luca Montessoro

    Creation date:    29 January 1988

    Purpose:          memory allocation

 ******************************************************************************/


#ifdef malloc
#undef malloc


#define MEM_POOL_SIZE  (unsigned long) 0xFFFF  /* small, too small!!! */


char           mem_pool_1 [MEM_POOL_SIZE];
unsigned long  mem_position_1 = 0;

#if 0
char           mem_pool_2 [MEM_POOL_SIZE];
unsigned long  mem_position_2 = 0;
char           mem_pool_3 [MEM_POOL_SIZE];
unsigned long  mem_position_3 = 0;
char           mem_pool_4 [MEM_POOL_SIZE];
unsigned long  mem_position_4 = 0;
#endif


char  *no_malloc (size)

unsigned long  size;

{

   /* local variable */

   unsigned long  save_mem_position;


   /* beginning of executable part */

   if ((mem_position_1 + size) <= MEM_POOL_SIZE)
   {
      save_mem_position = mem_position_1;
      mem_position_1 += size;
      return (mem_pool_1 + save_mem_position);
   }

#if 0

   if ((mem_position_2 + size) <= MEM_POOL_SIZE)
   {
      save_mem_position = mem_position_2;
      mem_position_2 += size;
      return (mem_pool_2 + save_mem_position);
   }

   if ((mem_position_3 + size) <= MEM_POOL_SIZE)
   {
      save_mem_position = mem_position_3;
      mem_position_3 += size;
      return (mem_pool_3 + save_mem_position);
   }

   if ((mem_position_4 + size) <= MEM_POOL_SIZE)
   {
      save_mem_position = mem_position_4;
      mem_position_4 += size;
      return (mem_pool_4 + save_mem_position);
   }

#endif

   return ((char *) NULL);

}  /* end of no_malloc */


#endif



#if 0

#ifdef malloc
#undef malloc

char  *no_malloc (size)

unsigned long  size;

{

   /* extern function */

   extern void  *malloc();


   /* beginning of executable part */

   if (size >= (unsigned long) 0xFFFF)  return ((char *) NULL);

   return ((char *) malloc ((size_t) size));

}  /* end of no_malloc */


#endif

#endif




/******************************************************************************

    Function name:    no_free

    Authors:          Pier Luca Montessoro

    Creation date:    29 January 1988

    Purpose:          release the allocated memory

 ******************************************************************************/


#ifdef free
#undef free


no_free (ptr)

char  *ptr;

{

   /* beginning of executable part */

   /* the memory is never released in the current implementation for 
      16-bits machines */

   return;

}  /* end of no_free */


#endif



/****************************************************************************** 

    Function name:    no_cfree

    Author:           Pier Luca Montessoro

    Creation date:    19 January 1988

    Purpose:          cfree() function emulation

 ******************************************************************************/


#ifdef  cfree
#undef  cfree


no_cfree (ptr)

char  *ptr;

{

   /* external function */

   extern void  free();


   /* beginning of executable part */

   free (ptr);


   return;

}  /* end of no_cfree */


#endif



/****************************************************************************** 

    Function name:    no_times

    Author:           Pier Luca Montessoro

    Creation date:    19 January 1988

    Purpose:          times() function emulation

 ******************************************************************************/


#ifdef  times
#undef  times


static long start_time = 0;     /* it stores the absolute CPU time of   */
                                /* first evocation of the function      */


no_times (present)

struct times_buffer        /* this structure contains the information */
{                          /* returned by the O.S. It is composed of  */
   long proc_user_time;    /* of four words containing system and     */
   long proc_system_time;  /* user, process and children times.       */
   long child_user_time;   /* For more information see:               */
   long child_system_time; /* - Programming in VAX-11 C, Digital,     */
}  *present;               /*   AA-L370A-TE, page 158, or             */
                           /* - Unix User Manual, TIMES(2)            */

{

   /* external function */

   extern clock_t  clock();


   /* beginning of executable part */

   if (start_time == 0)
   {
      /* First evokation: call clock() to fill present start_time */
      start_time = (100 * (long) clock()) / CLK_TCK;
      present->proc_user_time = 0;
   }
   else
   {
      /* Computes the elapsed user process time */
      present->proc_user_time = (100 * (long) clock()) / CLK_TCK - start_time;
   }

   present->proc_system_time = 0;
   present->child_user_time = 0;
   present->child_system_time = 0;

   return;

}  /* end of no_times */


#endif



/****************************************************************************** 

    Function name:    no_tolower

    Author:           Pier Luca Montessoro

    Creation date:    8 June 1988

    Purpose:          tolower() function emulation

 ******************************************************************************/


#ifdef  tolower
#undef  tolower


char  no_tolower (c)

char  c;

{

   return ((c) >= 'A' && (c) <= 'Z' ? ((c) | 0x20) : (c));

}  /* end of no_tolower */


#endif



/****************************************************************************** 

    Function name:    no_toupper

    Author:           Pier Luca Montessoro

    Creation date:    8 June 1988

    Purpose:          toupper() function emulation

 ******************************************************************************/


#ifdef  toupper
#undef  toupper


char  no_toupper (c)

char  c;

{

   return ((c) >= 'a' && (c) <= 'z' ? ((c) & 0x5F) : (c));

}  /* end of no_toupper */


#endif



/****************************************************************************** 

    Function name:    no_sleep

    Author:           Pier Luca Montessoro

    Creation date:    19 January 1988

    Purpose:          sleep() function emulation

 ******************************************************************************/


#ifdef sleep
#undef sleep

/* yet to be implemented... */


#endif


